require 'erb'
require 'nokogiri'
require 'rspec/core/formatters/console_codes'

module Manacle
  class Example
    attr_reader :description, :full_description, :execution_result, :run_time, :status, :metadata, :exception, :file_path, :last_response, :case, :params

    def initialize(example, last_response)
      @description = example.description
      @full_description = build_description(example)
      @execution_result = example.execution_result
  #    @run_time = RSpec::Core::Formatters::Helpers.format_duration(@execution_result.run_time)
      @run_time = execution_result.run_time
      @status = @execution_result.status.to_s
      @metadata = example.metadata
      @exception = example.exception
      @file_path = @metadata[:file_path]
      @last_response = last_response if ['passed', 'failed'].include?(@status)
      # @last_response = last_response
      @case = @metadata[:case]
      @params = @metadata[:params]
      @params ||= build_params(example)
    end
 
    def build_params(example)
      return [] if last_response.nil?
      xml = Nokogiri::XML(last_response.request.options[:body])

      ns = {ns: 'http://www.quantiguous.com/services'}
      params = []

      example.example_group.described_class.param_defaults.each do |pv|
        p = pv.keys.first; v = pv.values.first
        if v.nil?
           params << xml.at_xpath("//ns:#{p}", ns).name.to_sym unless xml.at_xpath("//ns:#{p}", ns).nil?
        else
           params << xml.at_xpath("//ns:#{p}", ns).name.to_sym unless xml.at_xpath("//ns:#{p}", ns).try(:text) == v
        end
      end
      params
    end

    def build_description(example)
      full_description = [example.description, example.example_group.description]
      parent = example.example_group.parent
      while parent.is_a?(Class)
        full_description << parent.try(:description)
        parent = parent.parent
      end
      full_description.reverse.join('<br/>')
    end
  
    def param_value(param)
      @reqDoc ||= Nokogiri::XML(@last_response.request.options[:body])
      @reqDoc.at_xpath('//ns:' + param.to_s, 'ns' => 'http://www.quantiguous.com/services').try(:text)
    end
  end

  class ExampleList
    include ERB::Util
    attr_reader :examples

    def initialize
      @examples = []
    end
  
    def add(example, last_response)
      @examples << Example.new(example, last_response)
    end

    def pretty_xml(xml)
     Nokogiri::XML(xml,&:noblanks).to_xml unless xml.nil?
    end

    def render
      erb = ERB.new(File.read(File.expand_path('report.erb',__dir__)))
      erb.result(binding)
    end
  end

  class Formatter 
    # This registers the notifications this formatter supports, and tells
    # us that this was written against the RSpec 3.x formatter API.
    RSpec::Core::Formatters.register self, :start, :stop, :example_started, :example_passed, :example_failed, :example_pending, :example_finished

    def initialize(output)
      @output = output
    end

    def start(notification)
      @examples = ExampleList.new
      @count = 0
      @passed_count = 0
      @failed_count = 0
      @pending_count = 0
    end

    def stop(notification)
      @output << @examples.render
    end

    def example_started(notification)
      @count += 1
    end

    def example_passed(notification)
      @passed_count += 1
    end

    def example_failed(notification)
      @failed_count += 1
    end

    def example_pending(notification)
      @pending_count += 1
    end

    def example_finished(notification)
      @examples.add(notification.example, ApiBanking::SoapClient.last_response)
    end
  end
end
