require 'rspec/expectations'

module Manacle
  RSpec::Matchers.define :fail_with do |fault_code, fault_subCode|
    define_method :is_fault? do |actual|
      actual.instance_of?(ApiBanking::Fault)
      return false unless actual.instance_of?(ApiBanking::Fault)
      unless fault_code.nil?
        return false if actual.code != fault_code
      end

      unless fault_subCode.nil?
        return false if actual.subCode != fault_subCode
      end

      return true
    end

    match do |actual|
      is_fault?(actual)
    end

    failure_message do |actual|
      if actual.instance_of?(ApiBanking::Fault)
        if actual[:code] != fault_code
          "expected that result would be #{fault_code} instead of #{actual[:code]} "
        else
          "expected that result would be #{fault_subCode} instead of #{actual[:subCode]} "
        end
      else
        "expected that result would be #{fault_code} instead of #{actual.class} with status code #{actual.transactionStatus.statusCode}"
      end
    end

    failure_message_when_negated do |actual|
      "expected that #{actual.transactionStatus.statusCode} would not be #{exp_status_code}"
    end
  end
end
