require 'active_record'
require 'factory_girl'

module Manacle
  module_function
  def setup(service_name)
     ActiveRecord::Base.establish_connection(
       adapter: "oracle_enhanced",
       database: ENV['DATABASE'],
       username: ENV['DB_USERNAME'],
       password: ENV['DB_PASSWORD']
     )

     [:API_USERNAME, :API_PASSWORD, :DB_USERNAME, :DB_ADAPTER, :DATABASE, :DB_PASSWORD, :SERVICE_URL, :CONSUMER_KEY, :CONSUMER_SECRET].each do |e|
        raise "required ENV variable #{e} not set" if ENV[e.to_s].nil?
     end

     RSpec.configure do |c|
       c.include FactoryGirl::Syntax::Methods

       c.default_formatter = Manacle::Formatter

       c.before(:suite) do

         Typhoeus::Config.verbose = true unless ENV['VERBOSE'].nil?

         Kernel.const_get("ApiBanking::#{service_name}").configure do |config|
           url = ApiBanking::Environment::QG::DEMO.new(ENV['API_USERNAME'], ENV['API_PASSWORD'])
           url.endpoints[service_name.to_sym] = ENV['SERVICE_URL']
           config.environment = url

           puts "service #{service_name}"
           puts "connecting to #{ENV['SERVICE_URL']}"

           unless ENV['SERVICE_PROXY'].nil?
              puts "using proxy at #{ENV['SERVICE_PROXY']}"
              config.proxy = ENV['SERVICE_PROXY']
           end
         
         end

         Manacle.const_set(:DATA, YAML.load(File.open("#{Dir.pwd}/spec/config/data.yml")))

         Dir["#{Dir.pwd}/models/**/*.rb"].each { |f| require f }

         FactoryGirl.find_definitions

       end
     end
  end
end
